import os, pygame, random
from pygame.locals import *

from projectiles import *

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

class AlienEnemy(pygame.sprite.Sprite):
    """ An alien enemy, beware! """
    def __init__(self, xy, floor):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_image('alienEnemy_stand1.gif', -1)
        self.stand_image = self.image
        self.stand_imageFlip = pygame.transform.flip(self.stand_image, 1, 0)

        self.runLeft = []
        runLeft1, null = load_image('ninja_walk1.gif', -1)
        runLeft2, null = load_image('ninja_walk2.gif', -1)
        runLeft3, null = load_image('ninja_walk3.gif', -1)
        self.runLeft.append(runLeft1)
        self.runLeft.append(runLeft2)
        self.runLeft.append(runLeft3)
        self.runLeft.append(runLeft2)

        self.runRight = []
        runRight1 = pygame.transform.flip(runLeft1, 1, 0)
        runRight2 = pygame.transform.flip(runLeft2, 1, 0)
        runRight3 = pygame.transform.flip(runLeft3, 1, 0)
        self.runRight.append(runRight1)
        self.runRight.append(runRight2)
        self.runRight.append(runRight3)
        self.runRight.append(runRight2)

        self.cFrame = 0
        self.rect.topleft = xy
        self.level = floor
        self.facingRight = False
        self.life = 100
        self.vision = 25  # how well can we see in the dark?

        self.aiState = 0 # standing
        self.aiCounter = 0
        self.aiCounterMax = 30
        self.shootCounter = 60 # allows for a timer
                               # when shooting, so we don't shoot too often
        self.shootCounterMax = self.shootCounter

        self.aiPanicCounter = 0
        self.aiPanicCounterMax = 360
        # setup our guarding post.
        self.aiGuardXMin = 0
        self.aiGuardXMax = 0
        initialX, initialY = xy
        if (initialX - 64) < 0: # if our guard is too close to the left side
            self.aiGuardXMin = 0
            self.aiGuardXMax = initialX + 128
        elif (initialX + 64) > 608: # if our guard is too close to the right side
            self.aiGuardXMax = 640
            self.aiGuardXMin = initialX - 128
        else:                       # not too close to either, so lets roll
            self.aiGuardXMin = initialX - 64
            self.aiGuardXMax = initialX + 64

    def update(self, player, floorLights, floorOn, effectList):
        if self.shootCounter < self.shootCounterMax:
            self.shootCounter = self.shootCounter + 1
        
        if player.level == self.level:
            if self.shootCounter == self.shootCounterMax: # only shoot when we have bullets
                if floorOn == 1:
                    playerX, playerY = player.rect.topleft
                    selfX, selfY = self.rect.topleft
                    if self.facingRight == False:
                        if selfX > playerX:
                            self.attack(effectList)
                    elif self.facingRight == True:
                        if selfX < playerX:
                            self.attack(effectList)
                else:
                    playerX, playerY = player.rect.center
                    selfX, selfY = self.rect.topleft
                    if self.facingRight == False:
                        if selfX > playerX:
                            lightArea = playerX / 128 # gives us where the player is
                            lampX = int(floorLights[lightArea])
                            if lampX > 0:
                                lampPlace = ((130*lightArea+int(lampX)*10)-7)
                                if ( playerX > lampPlace - self.vision ) and ( playerX < lampPlace + self.vision ):
                                    self.attack(effectList)
                    elif self.facingRight == True:
                        if selfX < playerX:
                            lightArea = playerX / 128 # gives us where the player is
                            lampX = int(floorLights[lightArea])
                            if lampX > 0:
                                lampPlace = ((130*lightArea+int(lampX)*10)-7)
                                if ( playerX > lampPlace - self.vision ) and ( playerX < lampPlace + self.vision ):
                                    self.attack(effectList)
                    
        self.aiCounter = self.aiCounter + 1
        if self.aiCounter > self.aiCounterMax:
            self.aiCounter = 0
            self.aiState = random.randrange(0,3,1) # random state 0-2
            self.aiCounterMax = random.randrange(15,45,1)
        if self.aiState == 1:       # move forward
            if self.facingRight == False:
                x,y = self.rect.topleft
                x = x - 2
                self.rect.topleft = (x,y)
                if ( x < self.aiGuardXMin ):
                    self.turnDirection()
            else:
                x,y = self.rect.topleft
                x = x + 2
                self.rect.topleft = (x,y)
                if ( x > self.aiGuardXMax ):
                    self.turnDirection()
        elif self.aiState == 2:       # turn direction
            self.turnDirection()
            self.aiState = 0        # change back to standing
        elif self.aiState == 3:         # spotted the enemy!
            null = 0

    def turnDirection(self):
        self.facingRight = not self.facingRight
        if self.facingRight == True:
            self.image = self.stand_imageFlip
        else:
            self.image = self.stand_image

    def hit(self, damage, spriteList):
        self.life = self.life - damage
        if self.life <= 0:
            spriteList.remove(self)
            return 500
        else:
            return 55

    def attack(self, effectList):
        selfX, selfY = self.rect.topleft
        selfY = selfY + 20
        if self.facingRight == True:  # shoot bullet from right side
            selfX = selfX + 32
        effectList.add(Bullet((selfX,selfY), 5.0, self.facingRight, 0))
        self.shootCounter = 0

class DogEnemy(pygame.sprite.Sprite):
    """ A Doggy enemy, beware! """
    def __init__(self, xy, floor, face):
        pygame.sprite.Sprite.__init__(self)
        self.image, self.rect = load_image('dog_walk1.gif', -1)
        self.stand_image = self.image
        self.stand_imageFlip = pygame.transform.flip(self.image, 1, 0)

        self.cFrame = 0
        self.rect.topleft = xy
        self.level = floor
        self.facingRight = face
        if self.facingRight == True:
            self.image = self.stand_imageFlip
        self.life = 200     # more life!

        self.aiState = 0 # 0 = waiting, 1 = seeking, 2 = pacing, 3 = turning, 4 = sitting
        self.aiCounter = 0
        self.aiCounterMax = 30

        self.aiPanicCounter = 0
        self.aiPanicCounterMax = 360


    def update(self, player, floorLights, floorOn, effectList):

        if player.level == self.level:
            if floorOn == 1:
                self.aiState = 1 # seek! go go go
            else:
                playerX, playerY = player.rect.center
                selfX, selfY = self.rect.center
                if self.facingRight == False:
                    if selfX > playerX:
                        if selfX - playerX < 200:
                            self.aiState = 1
                elif self.facingRight == True:
                    if selfX < playerX:
                        if playerX - selfX < 200:
                            self.aiState = 1
        elif player.level != self.level: # if player has left the level, roam
            if self.aiState != 0:   #gotta get out of wait mode first
                self.aiCounter = self.aiCounter + 1
                if self.aiCounter > self.aiCounterMax:
                    self.aiCounter = 0
                    self.aiState = random.randrange(2,5,1) # random state either 2-4
                    self.aiCounterMax = random.randrange(15,45,1)
        if self.aiState == 1:       # seek!
            playerX, playerY = player.rect.center
            selfX, selfY = self.rect.center
            if selfX < playerX:
                if self.facingRight == False:
                    self.turnDirection()
            elif selfX > playerX:
                if self.facingRight == True:
                    self.turnDirection()
            if self.facingRight == False:
                selfX = selfX - 2   # run Fast!
                self.rect.center = (selfX,selfY)
            elif self.facingRight == True:
                selfX = selfX + 2
                self.rect.center = (selfX,selfY)
        elif self.aiState == 2:       # Pace
            if self.facingRight == False:
                x,y = self.rect.topleft
                x = x - 2
                self.rect.topleft = (x,y)
                if x < 5:
                    self.turnDirection()
            elif self.facingRight == True:
                x,y = self.rect.topleft
                x = x + 2
                self.rect.topleft = (x,y)
                if x > 571:
                    self.turnDirection() # 640 - 64 width - 5 extra
        elif self.aiState == 3:
            self.turnDirection()
            self.aiState = 4    # sit! aka nothing
        elif self.aiState == 4:
            null = 0 # change image to sitting?

    def turnDirection(self):
        self.facingRight = not self.facingRight
        if self.facingRight == True:
            self.image = self.stand_imageFlip
        else:
            self.image = self.stand_image

    def hit(self, damage, spriteList):
        self.life = self.life - damage
        if self.life <= 0:
            spriteList.remove(self)
            return 1000
        else:
            self.aiState = 1
            return 35
