import os, pygame
from pygame.locals import *

#functions to create our resources
def load_image(name, colorkey=None):
    fullname = os.path.join('Data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('Music', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound

#classes for our game objects
class Ninja(pygame.sprite.Sprite):
    """Control your Ninja!"""
    def __init__(self, xy):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = load_image('ninja_stand1.gif', -1)
        self.stand_image = self.image
        self.stand_imageFlip = pygame.transform.flip(self.stand_image, 1, 0)

        self.runLeft = []
        runLeft1, null = load_image('ninja_walk1.gif', -1)
        runLeft2, null = load_image('ninja_walk2.gif', -1)
        runLeft3, null = load_image('ninja_walk3.gif', -1)
        self.runLeft.append(runLeft1)
        self.runLeft.append(runLeft2)
        self.runLeft.append(runLeft3)
        self.runLeft.append(runLeft2)

        self.runRight = []
        runRight1 = pygame.transform.flip(runLeft1, 1, 0)
        runRight2 = pygame.transform.flip(runLeft2, 1, 0)
        runRight3 = pygame.transform.flip(runLeft3, 1, 0)
        self.runRight.append(runRight1)
        self.runRight.append(runRight2)
        self.runRight.append(runRight3)
        self.runRight.append(runRight2)
        self.cFrame = 0

        self.aimLeft = []
        aimLeft1, null = load_image('ninja_aim1.gif', -1)
        aimLeft2, null = load_image('ninja_aim2.gif', -1)
        aimLeft3, null = load_image('ninja_aim3.gif', -1)
        self.aimLeft.append(aimLeft1)
        self.aimLeft.append(aimLeft2)
        self.aimLeft.append(aimLeft3)

        self.aimRight = []
        aimRight1 = pygame.transform.flip(aimLeft1, 1, 0)
        aimRight2 = pygame.transform.flip(aimLeft2, 1, 0)
        aimRight3 = pygame.transform.flip(aimLeft3, 1, 0)
        self.aimRight.append(aimRight1)
        self.aimRight.append(aimRight2)
        self.aimRight.append(aimRight3)

        self.gunNoise = load_sound('playergunNoise.wav')
        self.aimState = 0
        self.currentWeapon = 0 # 0 = gun, 1 = sword
        
        self.rect.topleft = xy
        self.movement = 0 #standing
        self.face = 0 # facing left
        self.life = 100

        self.level = 0

        self.score = 0 # players currentScore
        self.lives = 3 # player lives count

    def update(self):
        "move the fist based on the mouse position"
        if self.aimState == 1:
            if self.cFrame < 14:
                self.cFrame = self.cFrame + 1
            if self.face == 0:
                if self.image != self.aimLeft[self.cFrame / 5]:
                    self.image = self.aimLeft[self.cFrame / 5]
            elif self.face == 1:
                if self.image != self.aimRight[self.cFrame / 5]:
                    self.image = self.aimRight[self.cFrame / 5]
        elif self.movement == 0: # standing
            if self.face == 0:
                self.image = self.stand_image
            elif self.face == 1:
                self.image = self.stand_imageFlip
        elif self.movement == 1: # move left
            x,y = self.rect.topleft
            if x > 5:
                x = x - 3
                self.rect.topleft = (x,y)
            if self.cFrame < 39:
                self.cFrame = self.cFrame + 1
            else:
                self.cFrame = 0
            self.image = self.runLeft[self.cFrame / 10]
        elif self.movement == 2: # move right
            x,y = self.rect.topleft
            if x < 603:
                x = x + 3
                self.rect.topleft = (x,y)
            if self.cFrame < 39:
                self.cFrame = self.cFrame + 1
            else:
                self.cFrame = 0
            self.image = self.runRight[self.cFrame / 10]
            

    def moveLeft(self):
        if self.aimState == 0:
            self.movement = 1
            self.cFrame = 0
        self.face = 0

    def moveRight(self):
        if self.aimState == 0:
            self.movement = 2
            self.cFrame = 0
        self.face = 1

    def stop(self):
        self.movement = 0

    def aim(self):
        if self.aimState == 0:
            print "Aim State ON!"
            self.aimState = 1
            self.movement = 0
            self.cFrame = 0
        elif self.aimState == 1:
            print "Aim State OFF!"
            self.aimState = 0

    def switchWeapon(self):
        if self.currentWeapon == 0:
            self.currentWeapon = 1
            return self.currentWeapon
        elif self.currentWeapon == 1:
            self.currentWeapon = 0
            return self.currentWeapon

    def attack(self):
        # what the self.cFrame check does is make sure the player can't
        # attack while you are beginning your aim!
        if self.aimState == 1:
            if self.cFrame == 14:
                if self.currentWeapon == 0:
                    self.gunNoise.play()
                    if self.face == 0:
                        return self.rect.topleft, "shootLeft"
                    elif self.face == 1:
                        return self.rect.topleft, "shootRight"
                elif self.currentWeapon == 1:
                    if self.face == 0:
                        return self.rect.topleft, "swordLeft"
                    elif self.face == 1:
                        return self.rect.topleft, "swordRight"
            else:
                return (0,0), "nothing"
        else:
            return (0,0), "nothing"

    def stopAttack(self):
        null = 0        # for rapid firing!

    def checkAction(self, action):
        actionName, actionVar = action
        if actionName == "warp":
            self.rect.topleft = actionVar
            x,y = actionVar
            self.level = (y - 42) / 64
            print "Warping now capt'n! We're now on Level " + str(self.level)
        elif actionName == "ladder":
            print "Ladder at " + str(actionVar)
            x,y = self.rect.topleft
            y = y - 64
            self.rect.topleft = (x,y)
            self.level = self.level - 1
        elif actionName == "downLadder":
            print "Below Ladder at " + str(actionVar)
            x,y = self.rect.topleft
            y = y + 64
            self.rect.topleft = (x,y)
            self.level = self.level + 1
        elif actionName == "myExit":
            print "Exit at " + str(actionVar)
        elif actionName == "Complete":
            print "Completed Level at " + str(actionVar)
        elif actionName == "controlBox":
            print "Control Box at " + str(actionVar)
        elif actionName == "deadBox":
            print "Dead Box at " + str(actionVar)

    def damage(self, damage, knockBack):
        self.life = self.life - damage
        print "Life = " + str(self.life)
        if self.life > 0 and knockBack == True:
            if self.face == 0:
                x,y = self.rect.topleft
                if x < 605:
                    x = x + 7
                    self.rect.topleft = x,y
            elif self.face == 1:
                x,y = self.rect.topleft
                if x > 10:
                    x = x - 7
                    self.rect.topleft = x,y
        elif self.life <= 0:
            self.lives = self.lives - 1
        return self.life

    def levelReset(self,xy):
        self.cFrame = 0
        self.aimState = 0
        self.currentWeapon = 0
        self.rect.topleft = xy
        self.movement = 0
        self.face = 0
        self.life = 100
        self.level = 0

    def deadLife(self):
        if self.life > 0:
            return False
        else:
            return True

    def currentLife(self):
        return self.lives

    def stillPlaying(self):
        if self.lives > 0:
            return True
        else:
            return False

    def addScore(self, score):
        self.score = self.score + score
        print "Current Score = " + str(self.score)
        return self.score
        
        
